/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozCmgui
 *
 * The Initial Developer of the Original Code is UniServices Ltd
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): UniServices Ltd, Andrew Millar
 *                 Carey Stevens
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


function CORBARunner(f)
{
  this.f = f;
  return;
}

CORBARunner.prototype =
{
 QueryInterface: function(aIID)
 {
    if (
        aIID.equals(Components.interfaces.nsIRunnable) ||
        aIID.equals(Components.interfaces.nsISupports)
       )
    {
      return this;
    }
    throw Components.results.NS_NOINTERFACE;
 },

 run: function()
 {
   netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
   this.f();
 }
};


function  mozCmguiRunSafe(f)
{
  netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
  return f();
}



function mozCmguiResizeMonitor(sv, p)
{
  this.sceneViewer = sv;
  this.plugin = p;
  return this;
}


mozCmguiResizeMonitor.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.mozIPluginResizeMonitor) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  onResize: function(width, height)
  {
    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    this.sceneViewer.setWindowSize(width, height);
  }
};



function mozCmguiProxyToMain(object, iface)
{
  var eqs = Components.classes["@mozilla.org/event-queue-service;1"]
                      .getService(Components.interfaces.nsIEventQueueService);
  var main_thread = eqs.getSpecialEventQueue(Components.interfaces.nsIEventQueueService.UI_THREAD_EVENT_QUEUE);

  var proxyObjectManager = Components.classes["@mozilla.org/xpcomproxy;1"]
                                     .getService(Components.interfaces.nsIProxyObjectManager);

  const PROXY_SYNC = 0x01;
  const PROXY_ALWAYS = 0x04;
  return proxyObjectManager.getProxyForObject(main_thread, iface, object,
                                              PROXY_SYNC | PROXY_ALWAYS);
}



function mozCmguiDefineMemoryBlock(commandData, dm, contentURL, memoryName){
  if ((contentURL.indexOf("chrome://") != -1)
  || (contentURL.indexOf("http://") != -1)
  || (contentURL.indexOf("https://") != -1)
  || (contentURL.indexOf("ftp://") != -1)
  || (contentURL.indexOf("file://") != -1)) // contains protocol, so assume to be an absolute url already
  {
    var absoluteURL = contentURL;
  }
  else
  {
	  // make an absolute url using current directory as the base (not using
	  // href to ignore search and hash extensions)
    var url = window.location.protocol + "//" + window.location.host
		 + window.location.pathname;
    var absoluteDirectoryURL = url.substring(0, (url.lastIndexOf("/")+1));
    var absoluteURL = absoluteDirectoryURL + contentURL;
  }

  window.mcg.defineMemoryBlockFromUrl(commandData, dm, absoluteURL, memoryName);
}


function mozCmguiVersion()
{
  return Components.interfaces.mozICmgui.MAJOR_VERSION + '.' + Components.interfaces.mozICmgui.MINOR_VERSION  + '.' + Components.interfaces.mozICmgui.SUBMINOR_VERSION  + '.' + Components.interfaces.mozICmgui.BUILD_VERSION
}


function mozCmguiCheckValidVersion(minVersion, maxVersion)
{
  var min_version_array = minVersion.split(".");
  var max_version_array = maxVersion.split(".");

  var valid = true

  if (Components.interfaces.mozICmgui.MAJOR_VERSION 
	  < min_version_array[0])
  {
	  valid = false;
  }
  else if (Components.interfaces.mozICmgui.MAJOR_VERSION 
	  == min_version_array[0])
  {
	  if ((min_version_array.length > 1) && (min_version_array[1] != "*"))
	  {
		  if (Components.interfaces.mozICmgui.MINOR_VERSION 
			  < min_version_array[1])
		  {
			  valid = false;
		  } 
		  else if (Components.interfaces.mozICmgui.MINOR_VERSION 
			  == min_version_array[1])
		  {
			  if ((min_version_array.length > 2) && (min_version_array[2] != "*"))
			  {
				  if (Components.interfaces.mozICmgui.SUBMINOR_VERSION 
					  < min_version_array[2])
				  {
					  valid = false;
				  } 
				  else if (Components.interfaces.mozICmgui.SUBMINOR_VERSION 
					  == min_version_array[2])
				  {
					  if ((min_version_array.length > 3) && (min_version_array[3] != "*") &&
						  (Components.interfaces.mozICmgui.BUILD_VERSION 
							  < min_version_array[3]))
					  {
						  valid = false;
					  }
				  }
			  }
		  }
	  }
  }

  if (Components.interfaces.mozICmgui.MAJOR_VERSION 
	  > max_version_array[0])
  {
	  valid = false;
  }
  else if (Components.interfaces.mozICmgui.MAJOR_VERSION 
	  == max_version_array[0])
  {
	  if ((max_version_array.length > 1) && (max_version_array[1] != "*"))
	  {
		  if (Components.interfaces.mozICmgui.MINOR_VERSION 
			  > max_version_array[1])
		  {
			  valid = false;
		  } 
		  else if (Components.interfaces.mozICmgui.MINOR_VERSION 
			  == max_version_array[1])
		  {
			  if ((max_version_array.length > 2) && (max_version_array[2] != "*"))
			  {
				  if (Components.interfaces.mozICmgui.SUBMINOR_VERSION 
					  > max_version_array[2])
				  {
					  valid = false;
				  } 
				  else if (Components.interfaces.mozICmgui.SUBMINOR_VERSION 
					  == max_version_array[2])
				  {
					  if ((max_version_array.length > 3) && (max_version_array[3] != "*") &&
						  (Components.interfaces.mozICmgui.BUILD_VERSION 
							  > max_version_array[3]))
					  {
						  valid = false;
					  }
				  }
			  }
		  }
	  }
  }

  return (valid);
}

// Create a factory just so we don't have to modify the interface
function CommandDataFactory(mcg)
{
	this.mcg = mcg;
}

CommandDataFactory.prototype =
{
 createCommandData: function()
	{
		return (this.mcg.createCmguiCommandData());
	}
};

function mozCmguiInitialise(CmguiReadyObject)
{

	var mcg = Components.classes["@physiome.org/mozCmgui;1"]
		.createInstance(Components.interfaces.mozICmgui);

	window.mcg = mcg;

	var cdf = new CommandDataFactory(mcg);

	CmguiReadyObject.cmguiReady(cdf);

};

function mozCmguiGoToURLAndClose(url) {
  window.opener.open(url);
  window.close();
}

function mozCmguiGoToURL(url) {
  window.opener.open(url);
}
